import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import edu.uci.ics.jung.algorithms.importance.PageRank;
import edu.uci.ics.jung.algorithms.importance.Ranking;
import edu.uci.ics.jung.algorithms.importance.NodeRanking;
import edu.uci.ics.jung.algorithms.transformation.DirectionTransformer;
import edu.uci.ics.jung.graph.DirectedGraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.decorators.StringLabeller;
import edu.uci.ics.jung.graph.decorators.StringLabeller.UniqueLabelException;
import edu.uci.ics.jung.graph.filters.impl.NumericDecorationFilter;
import edu.uci.ics.jung.graph.impl.SparseGraph;
import edu.uci.ics.jung.io.PajekNetReader;
import edu.uci.ics.jung.utils.GraphUtils;
import edu.uci.ics.jung.visualization.FRLayout;
import edu.uci.ics.jung.visualization.Layout;
import edu.uci.ics.jung.visualization.VisualizationViewer;

/**
 *
 *
 * @author Scott White
 * @author Gonzalo Mena Mendoza
 *
 * Este programa utiliza software del Proyecto JUNG:
 *
 * Copyright (c) 2003 the JUNG Project and the Regents of the University of
 * California All rights reserved. JUNG is open-source under the BSD license;
 * see http://jung.sourceforge.net/license.txt for a description.
 */
public class Relevancia802 extends JPanel {
    private static final String TTULO = "Relevancia de los Estndares del IEEE 802 utilizando PageRank";

    public class PanelSimple extends JPanel {
        public PanelSimple(String ttulo) {
            super();
            setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
            Border borde     = BorderFactory.createLineBorder(Color.BLACK);
            Border compuesto = BorderFactory.createTitledBorder(borde, ttulo);
            setBorder(compuesto);
        }
    }

    private DirectedGraph       grafo;
    private Layout              disposicin;
    private PresentadorBsico   presentador;
    private VisualizationViewer visor;
    private JList               listaJuegos;
    private JSlider             deslizadorTamao;

    private static Map          archivosGrafos = new HashMap();
    static {
        archivosGrafos.put("Referencias Explcitas Internas",   "datos/802_refer_int.net");
        archivosGrafos.put("Menciones Simples Internas",        "datos/802_mencion_int.net");
    }

    public Relevancia802() throws HeadlessException {
        inicializar();
        redibujar();
        recalcular();
    }

    protected void inicializar( ) {
        // panel de juego de datos
        JPanel panelJuegos = new PanelSimple("Juego de Datos");
        listaJuegos = new JList(archivosGrafos.keySet().toArray());
        listaJuegos.setPreferredSize(new Dimension(180, 40));
        listaJuegos.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        listaJuegos.setLayoutOrientation(JList.VERTICAL);
        listaJuegos.setVisibleRowCount(1);
        listaJuegos.setSelectedIndex(0);
        listaJuegos.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                leerGrafo( archivosGrafos.get(listaJuegos.getSelectedValue()).toString() );
                recalcular();
                redibujar();
            }
        });
        panelJuegos.add(listaJuegos);

        // panel de tamao
        JPanel panelTamao = new PanelSimple("Tamao del Nodo");
        deslizadorTamao = new JSlider(JSlider.HORIZONTAL);
        deslizadorTamao.setPreferredSize(new Dimension(180, 40));
        deslizadorTamao.setPaintTicks(true);
        deslizadorTamao.setValue(0);
        deslizadorTamao.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                presentador.setSizeKey(PageRank.KEY);
                presentador.setNodeSizeScale(deslizadorTamao.getValue());
                repaint();
            }
        });
        panelTamao.add(deslizadorTamao);

        // botn de redibujar
        JButton botnRedibujar = new JButton("Redibujar");
        botnRedibujar.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                disposicin.restart();
                visor.prerelax();
                visor.repaint();
            }

        });

        // panel de control
        JPanel panelControl = new JPanel();
        panelControl.setPreferredSize(new Dimension(200, 400));
        panelControl.setLayout( new BoxLayout(panelControl, BoxLayout.PAGE_AXIS) );

        panelControl.add(panelJuegos);
        panelControl.add(panelTamao);
        panelControl.add(botnRedibujar);

        panelControl.revalidate();
        panelControl.repaint();

        // panel principal
        setLayout(new BorderLayout());
        add(panelControl, BorderLayout.EAST);

        // leer grafo inicial
        leerGrafo( archivosGrafos.get(listaJuegos.getSelectedValue()).toString() );
    }


    protected void redibujar() {
        disposicin = new FRLayout(grafo);
        presentador = new PresentadorBsico();
        presentador.setLabel("LABEL");
        presentador.setSizeKey(PageRank.KEY);
        if (visor != null) {
            remove(visor);
        }
        visor = new VisualizationViewer(disposicin, presentador);
        visor.setBackground(Color.WHITE);
        add(visor);
        visor.revalidate();
        visor.repaint();
    }

    protected void recalcular() {
        // evaluar relevancia utilizando PageRank
        PageRank pageRank = new PageRank(grafo, 0.15);
        pageRank.setRemoveRankScoresOnFinalize(false);
        pageRank.evaluate();
        List listaClasificacin = pageRank.getRankings();

        // mostrar en salida estndar
        //pageRank.printRankings(true, true);
        System.out.println(TTULO);
        System.out.println( listaJuegos.getSelectedValue() );
        System.out.println("Lugar \tPageRank        \tEntran \tSalen \tNombre del Estndar");
        Iterator iter = listaClasificacin.iterator();
        for (int i = 1; iter.hasNext(); ++i) {
            NodeRanking r = (NodeRanking) iter.next();
            Vertex v = r.vertex;
            System.out.println(
                i             + ".\t" +
                r.rankScore   + "\t" +
                v.inDegree()  + "\t" +
                v.outDegree() + "\t" +
                StringLabeller.getLabeller(grafo, PajekNetReader.LABEL).getLabel(v)
            );
        }
        System.out.println();

        // calcular lmites apropiados para el deslizador de tamao
        Ranking mximo = (Ranking) listaClasificacin.get(0);
        Ranking mnimo = (Ranking) listaClasificacin.get(listaClasificacin.size() - 1);
        int minScaleRatio = (int) Math.floor(1.0 / mnimo.rankScore);
        int maxScaleRatio = (int) Math.floor(1.0 / mximo.rankScore);
        deslizadorTamao.setMaximum(minScaleRatio * 30);
        deslizadorTamao.setMinimum(maxScaleRatio * 10);

        // redibujar
        visor.revalidate();
        visor.repaint();
    }

    void leerGrafo(String archivo) {
        try {
            // Utilizar el formato de PajekNet para grafos/redes
            PajekNetReader pnr = new PajekNetReader(true);

            // Construir grafo dirigido
            Graph tmp = pnr.load(archivo, new SparseGraph());
            grafo = DirectionTransformer.toDirected(tmp);

            // Etiquetar nodos del grafo
            StringLabeller tsl  = StringLabeller.getLabeller(tmp,   PajekNetReader.LABEL);
            StringLabeller gsl  = StringLabeller.getLabeller(grafo, PajekNetReader.LABEL);
            GraphUtils.copyLabels(tsl, gsl);
            }
        catch(IOException e) {
            //JOptionPane.showMessageDialog(this,
            //    "Error de lectura:\n\n" + .getMessage(), "ERROR", JOptionPane.ERROR_MESSAGE
            //);
        }
        catch(UniqueLabelException e) {
            //JOptionPane.showMessageDialog(this,
            //    "Etiqueta de nodo duplicada:\n\n" + .getMessage(), "ERROR", JOptionPane.ERROR_MESSAGE
            //);
        }
    }

    public static void main(String[] args) throws IOException, UniqueLabelException {
        // Instanciar panel
        Relevancia802 panelRelevancia = new Relevancia802();

        //  y ponerlo en una ventana
        JFrame ventana = new JFrame(TTULO);
        ventana.getContentPane().add(panelRelevancia);
        ventana.setSize(800, 600);
        ventana.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        ventana.pack();
        ventana.show();
    }

}
